import { ADDRESS_PATH, BOOTSTRAP_PATH, TERMINATE_PATH, INSTALL_PATH, LOCAL_PORT, PEER_ID_PATH, PROGRAMS_PATH, PROGRAM_PATH, RESTART_PATH, TRUST_PATH, } from "./routes.js";
import { multiaddr } from "@multiformats/multiaddr";
import { signRequest } from "./signes-request.js";
import { PublicSignKey, getPublicKeyFromPeerId, } from "@peerbit/crypto";
export const client = async (keypair, endpoint = "http://localhost:" + LOCAL_PORT) => {
    const isLocalHost = endpoint.startsWith("http://localhost");
    const { default: axios } = await import("axios");
    const axiosInstance = axios.create();
    axiosInstance.interceptors.request.use(async (config) => {
        const url = new URL(config.url);
        await signRequest(config.headers, config.method, url.pathname + url.search, config.data, keypair);
        return config;
    });
    const validateStatus = (status) => {
        return (status >= 200 && status < 300) || status == 404;
    };
    const throwIfNot200 = (resp) => {
        if (resp.status !== 200) {
            throw new Error(resp.data);
        }
        return resp;
    };
    const getBodyByStatus = (resp) => {
        if (resp.status === 404) {
            return;
        }
        if (resp.status == 200) {
            return resp.data;
        }
        throw new Error(typeof resp.data === "string" ? resp.data : resp.data.toString());
    };
    const getId = async () => throwIfNot200(await axiosInstance.get(endpoint + PEER_ID_PATH, {
        validateStatus,
        timeout: 5000,
    })).data;
    return {
        peer: {
            id: {
                get: getId,
            },
            addresses: {
                get: async () => {
                    return throwIfNot200(await axiosInstance.get(endpoint + ADDRESS_PATH, {
                        validateStatus,
                    })).data.map((x) => multiaddr(x));
                },
            },
        },
        trust: {
            add: async (key) => {
                const result = await axiosInstance.put(endpoint +
                    TRUST_PATH +
                    "/" +
                    encodeURIComponent(key instanceof PublicSignKey
                        ? key.hashcode()
                        : getPublicKeyFromPeerId(key).hashcode()), undefined, { validateStatus });
                if (result.status !== 200 && result.status !== 404) {
                    throw new Error(result.data);
                }
                return result.status === 200 ? true : false;
            },
            remove: async (key) => {
                const result = await axiosInstance.delete(endpoint +
                    TRUST_PATH +
                    "/" +
                    encodeURIComponent(key instanceof PublicSignKey
                        ? key.hashcode()
                        : getPublicKeyFromPeerId(key).hashcode()), { validateStatus });
                if (result.status !== 200 && result.status !== 404) {
                    throw new Error(result.data);
                }
                return result.status === 200 ? true : false;
            },
        },
        program: {
            has: async (address) => {
                const result = await axiosInstance.head(endpoint +
                    PROGRAM_PATH +
                    "/" +
                    encodeURIComponent(address.toString()), { validateStatus });
                if (result.status !== 200 && result.status !== 404) {
                    throw new Error(result.data);
                }
                return result.status === 200 ? true : false;
            },
            open: async (program) => {
                const resp = throwIfNot200(await axiosInstance.put(endpoint + PROGRAM_PATH, JSON.stringify(program), {
                    validateStatus,
                }));
                return resp.data;
            },
            close: async (address) => {
                throwIfNot200(await axiosInstance.delete(endpoint +
                    PROGRAM_PATH +
                    "/" +
                    encodeURIComponent(address.toString()), {
                    validateStatus,
                }));
            },
            drop: async (address) => {
                throwIfNot200(await axiosInstance.delete(endpoint +
                    PROGRAM_PATH +
                    "/" +
                    encodeURIComponent(address.toString()) +
                    "?delete=true", {
                    validateStatus,
                }));
            },
            list: async () => {
                const resp = throwIfNot200(await axiosInstance.get(endpoint + PROGRAMS_PATH, {
                    validateStatus,
                }));
                return resp.data;
            },
        },
        dependency: {
            install: async (instruction) => {
                const resp = await axiosInstance.put(endpoint + INSTALL_PATH, JSON.stringify(instruction), {
                    validateStatus,
                });
                if (resp.status !== 200) {
                    throw new Error(typeof resp.data === "string" ? resp.data : resp.data.toString());
                }
                return resp.data;
            },
        },
        network: {
            bootstrap: async () => {
                throwIfNot200(await axiosInstance.post(endpoint + BOOTSTRAP_PATH, undefined, {
                    validateStatus,
                }));
            },
        },
        restart: async () => {
            throwIfNot200(await axiosInstance.post(endpoint + RESTART_PATH, undefined, {
                validateStatus,
            }));
        },
        terminate: async () => {
            throwIfNot200(await axiosInstance.post(endpoint + TERMINATE_PATH, undefined, {
                validateStatus,
            }));
        },
    };
};
//# sourceMappingURL=client.js.map